package com.sample;

import javax.persistence.EntityManagerFactory;
import javax.persistence.Persistence;

import org.drools.KnowledgeBase;
import org.drools.KnowledgeBaseFactory;
import org.drools.builder.KnowledgeBuilder;
import org.drools.builder.KnowledgeBuilderFactory;
import org.drools.builder.KnowledgeBuilderResult;
import org.drools.builder.ResourceType;
import org.drools.builder.ResultSeverity;
import org.drools.io.ResourceFactory;
import org.drools.runtime.Environment;
import org.drools.runtime.EnvironmentName;
import org.junit.AfterClass;
import org.junit.Assert;
import org.junit.Before;
import org.junit.BeforeClass;

import bitronix.tm.TransactionManagerServices;
import bitronix.tm.resource.jdbc.PoolingDataSource;

public abstract class JbpmPersistenceTestCase {
    protected static PoolingDataSource dataSource;
    protected static KnowledgeBase kbase;

    protected EntityManagerFactory emf;
    public static String[] files;

    @BeforeClass
    public static void beforeClass() {
        // useMySQL();
        useHSQL();
    }

    public static void useHSQL() {
        dataSource = new PoolingDataSource();
        dataSource.setMaxPoolSize(3);
        dataSource.setAllowLocalTransactions(true);
        dataSource.setUniqueName("jdbc/jbpmDS");
        dataSource
                .setClassName("bitronix.tm.resource.jdbc.lrc.LrcXADataSource");
        dataSource.getDriverProperties().put("driverClassName",
                "org.hsqldb.jdbcDriver");
        dataSource.getDriverProperties().put("url",
                "jdbc:hsqldb:file:/home/jsvitak/bitronixDB");
        dataSource.init();
    }

    public static void useMySQL() {
        dataSource = new PoolingDataSource();
        dataSource.setMaxPoolSize(3);
        dataSource.setAllowLocalTransactions(true);
        dataSource.setUniqueName("jdbc/jbpmDS");
        dataSource
                .setClassName("bitronix.tm.resource.jdbc.lrc.LrcXADataSource");
        dataSource.getDriverProperties().put("driverClassName",
                "com.mysql.jdbc.Driver");
        dataSource
                .getDriverProperties()
                .put("url",
                        "jdbc:mysql://localhost:3306/jpa-test?user=jpa-test?password=jpa-test");
        dataSource.getDriverProperties().put("user", "jpa-test");
        dataSource.getDriverProperties().put("password", "jpa-test");
        dataSource.init();
    }

    @Before
    public void before() {
        System.setProperty("java.naming.factory.initial",
                "bitronix.tm.jndi.BitronixInitialContextFactory");
        emf = Persistence
                .createEntityManagerFactory("org.drools.persistence.jpa.bitronix");

        KnowledgeBuilder kbuilder = KnowledgeBuilderFactory
                .newKnowledgeBuilder();
        for (String file : files) {
            kbuilder.add(ResourceFactory.newClassPathResource(file),
                    ResourceType.BPMN2);
        }
        for (KnowledgeBuilderResult result : kbuilder.getResults(
                ResultSeverity.ERROR, ResultSeverity.WARNING)) {
            System.err.println(result.toString());
        }
        if (kbuilder.hasErrors())
            Assert.fail("Found errors in process!");

        kbase = KnowledgeBaseFactory.newKnowledgeBase();
        kbase.addKnowledgePackages(kbuilder.getKnowledgePackages());
    }

    /*
     * @Test public void test() { StatefulKnowledgeSession ksession =
     * kbase.newStatefulKnowledgeSession(); ProcessInstance pi =
     * ksession.startProcess("com.sample.process");
     * 
     * Assert.assertNull("ProcessInstance still present in database!",
     * ksession.getProcessInstance(pi.getId())); }
     * 
     * @Test public void testPersisted() { StatefulKnowledgeSession ksession =
     * JPAKnowledgeService.newStatefulKnowledgeSession(kbase, null,
     * getEnvironment()); int sessionId = ksession.getId(); ProcessInstance pi =
     * ksession.startProcess("com.sample.process");
     * 
     * ksession = JPAKnowledgeService.loadStatefulKnowledgeSession(sessionId,
     * kbase, null, getEnvironment());
     * Assert.assertNull("ProcessInstance still present in database!",
     * ksession.getProcessInstance(pi.getId())); }
     */

    protected Environment getEnvironment() {
        Environment result = KnowledgeBaseFactory.newEnvironment();
        result.set(EnvironmentName.ENTITY_MANAGER_FACTORY, emf);
        result.set(EnvironmentName.TRANSACTION_MANAGER,
                TransactionManagerServices.getTransactionManager());

        return result;
    }

    @AfterClass
    public static void afterClass() {
        if (dataSource != null)
            dataSource.close();
    }
}
